@@ -26,6 +26,7 @@ from komikku.utils import MISSING_IMG_RESOURCE_PATH
 class CardPage(Adw.NavigationPage):
     __gtype_name__ = 'CardPage'
 
+    toolbar_view = Gtk.Template.Child('toolbar_view')
     left_button = Gtk.Template.Child('left_button')
     filters_button = Gtk.Template.Child('filters_button')
     title_stack = Gtk.Template.Child('title_stack')
@@ -46,6 +47,7 @@ class CardPage(Adw.NavigationPage):
     chapters_selection_mode_clear_reset_button = Gtk.Template.Child('chapters_selection_mode_clear_reset_button')
     chapters_selection_mode_menubutton = Gtk.Template.Child('chapters_selection_mode_menubutton')
     info_scrolledwindow = Gtk.Template.Child('info_scrolledwindow')
+    backdrop_picture = Gtk.Template.Child('backdrop_picture')
     title_box = Gtk.Template.Child('title_box')
     cover_box = Gtk.Template.Child('cover_box')
     name_label = Gtk.Template.Child('name_label')
@@ -295,7 +297,7 @@ class CardPage(Adw.NavigationPage):
         if not self.pool_to_update:
             self.pool_to_update = True
             # Adjust revealer position
-            self.pool_to_update_revealer.set_margin_top(self.get_child().get_top_bar_height())
+            self.pool_to_update_revealer.set_margin_top(self.toolbar_view.get_top_bar_height())
             self.pool_to_update_revealer.props.transition_type = Gtk.RevealerTransitionType.NONE
             self.pool_to_update_revealer.set_reveal_child(True)
         else:
@@ -417,6 +419,8 @@ class CardPage(Adw.NavigationPage):
 
     def remove_backdrop(self):
         self.remove_css_class('backdrop')
+        self.backdrop_picture.set_paintable(None)
+        self.backdrop_picture.set_opacity(1)
 
     def set_actions_enabled(self, enabled):
         self.delete_action.set_enabled(enabled)
@@ -424,12 +428,28 @@ class CardPage(Adw.NavigationPage):
         self.sort_order_action.set_enabled(enabled)
 
     def set_backdrop(self):
-        if not self.manga or Adw.StyleManager.get_default().get_high_contrast() or not Settings.get_default().card_backdrop:
+        self.remove_backdrop()
+
+        method = Settings.get_default().card_backdrop_method
+        if not self.manga or Adw.StyleManager.get_default().get_high_contrast() or not method:
             return
 
-        if backdrop_colors_css := self.manga.backdrop_colors_css:
-            self.css_provider.load_from_string(backdrop_colors_css)
-            self.add_css_class('backdrop')
+        if method == 'blurred-cover':
+            if path := self.manga.backdrop_image_fs_path:
+                self.backdrop_picture.set_filename(path)
+
+                if info := self.manga.backdrop_info:
+                    if Adw.StyleManager.get_default().get_dark():
+                        opacity = 1 - info['luminance'][0]
+                    else:
+                        opacity = info['luminance'][1]
+                    self.backdrop_picture.set_opacity(opacity)
+
+        elif method == 'linear-gradient':
+            if css := self.manga.backdrop_colors_css:
+                self.css_provider.load_from_string(css)
+
+        self.add_css_class('backdrop')
 
     def set_unread_chapters_badge(self):
         # Show unread chapters (with Adw.ViewStackPage badge) if any
@@ -596,23 +616,23 @@ class InfoBox:
                 label.set_css_classes(['genre-label', 'caption'])
                 self.genres_wrapbox.append(label)
 
-            self.genres_wrapbox.get_parent().get_parent().set_visible(True)
+            self.genres_wrapbox.get_parent().set_visible(True)
         else:
-            self.genres_wrapbox.get_parent().get_parent().set_visible(False)
+            self.genres_wrapbox.get_parent().set_visible(False)
 
         if manga.scanlators:
             self.scanlators_label.set_markup(html_escape(', '.join(manga.scanlators)))
-            self.scanlators_label.get_parent().get_parent().set_visible(True)
+            self.scanlators_label.get_parent().set_visible(True)
         else:
-            self.scanlators_label.get_parent().get_parent().set_visible(False)
+            self.scanlators_label.get_parent().set_visible(False)
 
         self.chapters_label.set_markup(str(len(manga.chapters)))
 
         if manga.last_update:
             self.last_update_label.set_markup(manga.last_update.strftime(_('%m/%d/%Y')))
-            self.last_update_label.get_parent().get_parent().set_visible(True)
+            self.last_update_label.get_parent().set_visible(True)
         else:
-            self.last_update_label.get_parent().get_parent().set_visible(False)
+            self.last_update_label.get_parent().set_visible(False)
 
         self.set_disk_usage()
 
