@@ -1,17 +1,22 @@
-# Copyright (C) 2019-2024 Valéry Febvre
-# SPDX-License-Identifier: GPL-3.0-only or GPL-3.0-or-later
+# SPDX-FileCopyrightText: 2019-2025 Valéry Febvre
+# SPDX-License-Identifier: GPL-3.0-or-later
 # Author: Valéry Febvre <vfebvre@easter-eggs.com>
 
 import datetime
 from gettext import gettext as _
 import gc
 import importlib
+import json
 import logging
 import os
+from pathlib import Path
 import shutil
 import time
 
-from colorthief import ColorThief
+from modern_colorthief import get_palette
+from PIL import Image
+from PIL import ImageFilter
+from PIL import ImageStat
 
 from komikku.models.database import create_db_connection
 from komikku.models.database import insert_row
@@ -20,6 +25,8 @@ from komikku.models.database import update_rows
 from komikku.servers.utils import get_server_class_name_by_id
 from komikku.servers.utils import get_server_dir_name_by_id
 from komikku.servers.utils import get_server_module_name_by_id
+from komikku.utils import COVER_HEIGHT
+from komikku.utils import COVER_WIDTH
 from komikku.utils import get_cached_data_dir
 from komikku.utils import get_data_dir
 from komikku.utils import is_number
@@ -31,6 +38,7 @@ logger = logging.getLogger(__name__)
 
 class Manga:
     _chapters = None
+    _chapters_scanlators = None
     _server = None
 
     STATUSES = dict(
@@ -77,7 +85,7 @@ class Manga:
         data.update(dict(
             in_library=0,
             # Add fake last_read date: allows to display recently added manga at the top of the library
-            last_read=datetime.datetime.utcnow(),
+            last_read=datetime.datetime.now(datetime.UTC),
         ))
 
         # Long strip detection (Webtoon)
@@ -127,14 +135,13 @@ class Manga:
                 if '@define-color' not in data:
                     return data
 
-        try:
-            palette = ColorThief(cover_path).get_palette(color_count=2, quality=1)[:2]
-        except Exception:
+        palette = get_palette(cover_path, color_count=2)
+        if len(palette) != 2:
             # Single color image?
             return None
 
         colors = [':root {\n']
-        for index, color in enumerate(palette):
+        for index, color in enumerate(reversed(palette)):
             colors.append(f'\t--backdrop-background-color-{index}: rgb({color[0]} {color[1]} {color[2]} / 100%);\n')  # noqa: E702, E231
         colors.append('\t--backdrop-background-color-2: var(--window-bg-color);\n')
         colors.append('}\n')
@@ -144,6 +151,45 @@ class Manga:
 
         return ''.join(colors)
 
+    @property
+    def backdrop_image_fs_path(self):
+        if self.cover_fs_path is None:
+            return None
+
+        path = os.path.join(self.path, 'backdrop_image.jpg')
+        if os.path.exists(path):
+            return path
+
+        with Image.open(self.cover_fs_path) as image:
+            image = image.convert('RGB').filter(ImageFilter.GaussianBlur(35))
+            image.save(path, 'JPEG')
+
+        return path
+
+    @property
+    def backdrop_info(self):
+        if self.backdrop_image_fs_path is None:
+            return None
+
+        path = os.path.join(self.path, 'backdrop_info.json')
+        if os.path.exists(path):
+            with open(path) as fp:
+                return json.load(fp)
+
+        with Image.open(self.backdrop_image_fs_path) as image:
+            stat = ImageStat.Stat(image.convert('L'))
+            info = {
+                # Luninance values used to apply an opacity on Picture depending of color scheme (dark/light)
+                'luminance': [
+                    min((stat.mean[0] + stat.extrema[0][0]) / 510, 0.7),
+                    max((stat.mean[0] + stat.extrema[0][1]) / 510, 0.3),
+                ]
+            }
+            with open(path, 'w') as fp:
+                json.dump(info, fp)
+
+        return info
+
     @property
     def categories(self):
         db_conn = create_db_connection()
@@ -177,6 +223,34 @@ class Manga:
 
         return self._chapters
 
+    @property
+    def chapters_scanlators(self):
+        if self._chapters_scanlators is None:
+            db_conn = create_db_connection()
+
+            rows = db_conn.execute('SELECT DISTINCT scanlators, count(*) FROM chapters WHERE manga_id = ? GROUP BY scanlators', (self.id,))
+
+            scanlators = {}
+            for row in rows:
+                if not row[0]:  # None or []
+                    # Use 'Unknown' as virtual scanlator for chapters without scanlators defined
+                    row = (['Unknown'], row[1])
+
+                for scanlator in row[0]:
+                    if scanlator not in scanlators:
+                        scanlators[scanlator] = {
+                            'name': scanlator,
+                            'count': 0,
+                        }
+
+                    scanlators[scanlator]['count'] += row[1]
+
+            self._chapters_scanlators = list(scanlators.values()) or None
+
+            db_conn.close()
+
+        return self._chapters_scanlators
+
     @property
     def class_name(self):
         return get_server_class_name_by_id(self.server_id)
@@ -252,34 +326,11 @@ class Manga:
         return self._server
 
     def _save_cover(self, url):
-        if url is None:
-            return
-
-        # If cover has already been retrieved
-        # Check first if it has changed using ETag
-        current_etag = None
-        cover_etag_fs_path = os.path.join(self.path, 'cover.etag')
-        if os.path.exists(cover_etag_fs_path):
-            with open(cover_etag_fs_path, 'r') as fp:
-                current_etag = fp.read()
-
-        # Save cover image file
-        try:
-            cover_data, etag, _rtime = self.server.get_manga_cover_image(url, current_etag)
-        except Exception:
-            return
-        if cover_data is None:
-            return
-
-        cover_fs_path = os.path.join(self.path, 'cover.jpg')
-        with open(cover_fs_path, 'wb') as fp:
-            fp.write(cover_data)
-
-        if etag:
-            with open(cover_etag_fs_path, 'w') as fp:
-                fp.write(etag)
-        elif os.path.exists(cover_etag_fs_path):
-            os.remove(cover_etag_fs_path)
+        # Covers in landscape format are converted to portrait format
+        if self.server.save_image(url, self.path, 'cover', COVER_WIDTH, COVER_HEIGHT):
+            # Remove backdrop files (image, css, info)
+            for file in Path(self.path).glob('backdrop_*'):
+                os.unlink(file)
 
     def add_in_library(self):
         tmp_path = self.path
@@ -321,24 +372,62 @@ class Manga:
 
         db_conn = create_db_connection()
 
-        order = 'ASC' if direction == 1 else 'DESC'
         op = '>' if direction == 1 else '<'
+        order = 'ASC' if direction == 1 else 'DESC'
+        if self.filters and self.filters.get('scanlators'):
+            # Chapters must be filtered by scanlators (some scanlators are excluded)
+            excluded_scanlators = self.filters['scanlators']
+
+            # Subquery to get IDs of not filtered chapters
+            scanlators_subquery = f"""
+                SELECT DISTINCT c.id
+                FROM chapters c, json_each(scanlators)
+                WHERE json_each.value NOT IN ("{'", "'.join(excluded_scanlators)}") AND c.manga_id = {self.id}
+            """
+            if 'Unknown' not in excluded_scanlators:
+                # Add chapters without scanlators defined
+                scanlators_subquery += f"""
+                    UNION
+                    SELECT id FROM chapters WHERE (scanlators IS NULL OR scanlators->0 IS NULL) AND manga_id = {self.id}
+                """
+        else:
+            scanlators_subquery = None
 
         if self.sort_order in ('asc', 'desc', None):
-            row = db_conn.execute(
-                f'SELECT * FROM chapters WHERE manga_id = ? AND rank {op} ? ORDER BY rank {order}',
-                (self.id, chapter.rank)
-            ).fetchone()
+            if scanlators_subquery:
+                row = db_conn.execute(
+                    f'SELECT * FROM chapters WHERE manga_id = ? AND id IN ({scanlators_subquery}) AND rank {op} ? ORDER BY rank {order}',
+                    (self.id, chapter.rank)
+                ).fetchone()
+            else:
+                row = db_conn.execute(
+                    f'SELECT * FROM chapters WHERE manga_id = ? AND rank {op} ? ORDER BY rank {order}',
+                    (self.id, chapter.rank)
+                ).fetchone()
+
         elif self.sort_order in ('date-asc', 'date-desc'):
-            row = db_conn.execute(
-                f'SELECT * FROM chapters WHERE manga_id = ? AND date {op} ? ORDER BY date {order}, id {order}',
-                (self.id, chapter.date)
-            ).fetchone()
+            if scanlators_subquery:
+                row = db_conn.execute(
+                    f'SELECT * FROM chapters WHERE manga_id = ? AND id IN ({scanlators_subquery}) AND date {op} ? ORDER BY date {order}, id {order}',
+                    (self.id, chapter.date)
+                ).fetchone()
+            else:
+                row = db_conn.execute(
+                    f'SELECT * FROM chapters WHERE manga_id = ? AND date {op} ? ORDER BY date {order}, id {order}',
+                    (self.id, chapter.date)
+                ).fetchone()
+
         elif self.sort_order in ('natural-asc', 'natural-desc'):
-            row = db_conn.execute(
-                f'SELECT * FROM chapters WHERE manga_id = ? AND title {op} ? COLLATE natsort ORDER BY title {order}, id {order}',
-                (self.id, chapter.title)
-            ).fetchone()
+            if scanlators_subquery:
+                row = db_conn.execute(
+                    f'SELECT * FROM chapters WHERE manga_id = ? AND id IN ({scanlators_subquery}) AND title {op} ? COLLATE natsort ORDER BY title {order}, id {order}',
+                    (self.id, chapter.title)
+                ).fetchone()
+            else:
+                row = db_conn.execute(
+                    f'SELECT * FROM chapters WHERE manga_id = ? AND title {op} ? COLLATE natsort ORDER BY title {order}, id {order}',
+                    (self.id, chapter.title)
+                ).fetchone()
 
         db_conn.close()
 
@@ -508,9 +597,10 @@ class Manga:
                         logger.info('[UPDATE] {0} ({1}): Add new chapter {2}'.format(self.name, self.server_id, chapter_data['title']))
 
             if len(recent_chapters_ids) > 0 or nb_deleted_chapters > 0:
-                data['last_update'] = datetime.datetime.utcnow()
+                data['last_update'] = datetime.datetime.now(datetime.UTC)
 
             self._chapters = None
+            self._chapters_scanlators = None
 
             # Store old path
             old_path = self.path
