@@ -1,5 +1,5 @@
-# Copyright (C) 2019-2024 Valéry Febvre
-# SPDX-License-Identifier: GPL-3.0-only or GPL-3.0-or-later
+# SPDX-FileCopyrightText: 2019-2025 Valéry Febvre
+# SPDX-License-Identifier: GPL-3.0-or-later
 # Author: Valéry Febvre <vfebvre@easter-eggs.com>
 
 import logging
@@ -39,6 +39,8 @@ class KInfiniteCanvas(Gtk.Widget, Gtk.Scrollable):
         self.__hadj = None
         self.__vadj = None
 
+        self.vadjustment_value_changed_handler_id = None
+
         self.scroll_adjusting_delta = 0
         self.scroll_direction = None
         self.scroll_drag_offset = 0
@@ -89,12 +91,15 @@ class KInfiniteCanvas(Gtk.Widget, Gtk.Scrollable):
 
     @GObject.Property(type=Gtk.Adjustment)
     def vadjustment(self):
-        return self.__vadj
+        return self.__vadj or Gtk.Adjustment()
 
     @vadjustment.setter
     def vadjustment(self, adj):
-        self.vadjustment_value_changed_handler_id = adj.connect('value-changed', lambda adj: self.queue_allocate())
+        if not adj:
+            return
+
         self.__vadj = adj
+        self.vadjustment_value_changed_handler_id = adj.connect('value-changed', lambda _adj: self.queue_allocate())
 
     @GObject.Property(type=Gtk.ScrollablePolicy, default=Gtk.ScrollablePolicy.MINIMUM)
     def vscroll_policy(self):
@@ -137,6 +142,8 @@ class KInfiniteCanvas(Gtk.Widget, Gtk.Scrollable):
         first_page = self.get_first_child()
         last_page = self.get_last_child()
         if self.scroll_direction in (Gtk.DirectionType.DOWN, None):
+            self.scroll_direction = Gtk.DirectionType.DOWN
+
             if last_page.loadable and last_page._ic_position + last_page._ic_height < self.widget_height * (PRELOAD + 1):
                 self.emit('page-requested', 'end')
                 return
@@ -146,6 +153,8 @@ class KInfiniteCanvas(Gtk.Widget, Gtk.Scrollable):
                 return
 
         if self.scroll_direction in (Gtk.DirectionType.UP, None):
+            self.scroll_direction = Gtk.DirectionType.UP
+
             if first_page.loadable and first_page._ic_position > -self.widget_height * PRELOAD:
                 self.emit('page-requested', 'start')
                 return
@@ -154,10 +163,6 @@ class KInfiniteCanvas(Gtk.Widget, Gtk.Scrollable):
                 self.remove(last_page)
                 return
 
-        if self.scroll_direction is None:
-            # Set a default scroll direction at end of init
-            self.scroll_direction = Gtk.DirectionType.DOWN
-
     def append(self, page):
         """ Adds page at end """
         last_page = self.get_last_child()
@@ -186,9 +191,6 @@ class KInfiniteCanvas(Gtk.Widget, Gtk.Scrollable):
         self.queue_allocate()
 
     def configure_adjustments(self):
-        if self.vadjustment is None:
-            return
-
         lower = 0
         if first_page := self.get_first_child():
             if first_page.status == 'offlimit' and self.scroll_direction in (None, Gtk.DirectionType.UP):
@@ -216,7 +218,7 @@ class KInfiniteCanvas(Gtk.Widget, Gtk.Scrollable):
         self.scroll_adjusting_delta = 0
 
     def connect_signals(self):
-        if self.vadjustment:
+        if self.vadjustment and self.vadjustment_value_changed_handler_id is None:
             self.vadjustment_value_changed_handler_id = self.vadjustment.connect('value-changed', lambda adj: self.queue_allocate())
 
         # Keyboard navigation
@@ -224,7 +226,10 @@ class KInfiniteCanvas(Gtk.Widget, Gtk.Scrollable):
 
     def disconnect_signals(self):
         self.vadjustment.disconnect(self.vadjustment_value_changed_handler_id)
+        self.vadjustment_value_changed_handler_id = None
+
         self.pager.window.controller_key.disconnect(self.key_pressed_handler_id)
+        self.key_pressed_handler_id = None
 
     def dispose(self):
         self.disconnect_signals()
@@ -266,12 +271,13 @@ class KInfiniteCanvas(Gtk.Widget, Gtk.Scrollable):
             if not self.current_page_bottom and page._ic_position <= height and page._ic_position + page._ic_height > height:
                 self.current_page_bottom = page
 
+            visible = page._ic_position >= 0 and page._ic_position < height
+            visible |= page._ic_position + page_height >= 0 and page._ic_position + page_height < height
+            visible |= page._ic_position < 0 and page._ic_position + page_height >= height
+            page.obscured = not visible
+
             if page.status in ('rendering', 'allocable') and not page.activity_indicator.get_visible():
-                visible = page._ic_position >= 0 and page._ic_position < height
-                visible |= page._ic_position + page_height > 0 and page._ic_position + page_height <= height
-                visible |= page._ic_position < 0 and page._ic_position + page_height > height
-                if visible:
-                    page.activity_indicator.set_visible(True)
+                page.activity_indicator.set_visible(visible)
 
             position = Graphene.Point()
             position.init(0, page._ic_position)
