@@ -1,12 +1,10 @@
-# Copyright (C) 2019-2024 Valéry Febvre
-# SPDX-License-Identifier: GPL-3.0-only or GPL-3.0-or-later
+# SPDX-FileCopyrightText: 2019-2025 Valéry Febvre
+# SPDX-License-Identifier: GPL-3.0-or-later
 # Author: Valéry Febvre <vfebvre@easter-eggs.com>
 
 from abc import ABC
 from abc import abstractmethod
-from functools import cached_property
 import hashlib
-import inspect
 from io import BytesIO
 import json
 import logging
@@ -23,17 +21,17 @@ from http.cookiejar import CookieJar
 import requests
 
 from komikku.models.keyring import KeyringHelper
-from komikku.servers.loader import clear_servers_finders
 from komikku.servers.loader import ServerFinder
 from komikku.servers.loader import ServerFinderPriority
 from komikku.servers.utils import get_server_main_id_by_id
-from komikku.servers.utils import get_servers_modules
 from komikku.utils import BaseServer
 from komikku.utils import get_cache_dir
+from komikku.utils import get_cached_logos_dir
+from komikku.utils import LOGO_SIZE
 from komikku.utils import REQUESTS_TIMEOUT
 from komikku.utils import retry_session
 
-APP_MIN_VERSION = '1.60.0'  # Mininum app version required to use `Up-to-date servers modules`
+APP_MIN_VERSION = '1.79.0'  # Minimum app version required to use `Up-to-date servers modules`
 DOWNLOAD_MAX_DELAY = 1  # in seconds
 
 # https://www.localeplanet.com/icu/
@@ -46,6 +44,7 @@ LANGUAGES = dict(
     eo='Espéranto',
     es='Español',
     es_419='Español (Latinoamérica)',
+    fa='فارسی',
     fr='Français',
     it='Italiano',
     nl='Nederlands',
@@ -78,26 +77,20 @@ class Server(BaseServer, ABC):
     lang: str
 
     base_url = None
+    logo_url = None
 
     bypass_cf_url = None
     has_captcha = False
     has_cf = False
     has_login = False
-    headers = None
-    headers_images = None
-    http_client = 'requests'  # HTTP client
-    ignore_ssl_errors = False
     is_nsfw = False
     is_nsfw_only = False
     logged_in = False
     long_strip_genres = []
     manga_title_css_selector = None  # Used to extract manga title in a manga URL
-    status = 'enabled'
     sync = False
     true_search = True  # If False, hide search in Explorer search page (XKCD, DBM, pepper&carotte…)
 
-    __sessions = {}  # to cache all existing sessions
-
     @classmethod
     def get_manga_initial_data_from_url(cls, url):
         if cls.manga_title_css_selector:
@@ -149,11 +142,9 @@ class Server(BaseServer, ABC):
     def login(self, _username, _password):
         return False
 
-    @cached_property
+    @property
     def logo_path(self):
-        module_path = os.path.dirname(os.path.abspath(inspect.getfile(self.__class__)))
-
-        path = os.path.join(module_path, get_server_main_id_by_id(self.id) + '.png')
+        path = os.path.join(get_cached_logos_dir(), 'servers', f'{get_server_main_id_by_id(self.id)}.png')
         if not os.path.exists(path):
             return None
 
@@ -168,11 +159,11 @@ class Server(BaseServer, ABC):
             os.unlink(file_path)
 
         if all:
-            for id_ in Server.__sessions.copy():
+            for id_ in self._BaseServer__sessions.keys():
                 if id_.startswith(main_id):
-                    del Server.__sessions[id_]
-        elif self.id in Server.__sessions:
-            del Server.__sessions[self.id]
+                    del self._BaseServer__sessions[id_]
+        elif self.id in self._BaseServer__sessions:
+            del self._BaseServer__sessions[self.id]
 
     @abstractmethod
     def get_manga_data(self, initial_data):
@@ -297,6 +288,12 @@ class Server(BaseServer, ABC):
 
         return True
 
+    def save_logo(self):
+        return self.save_image(
+            self.logo_url, os.path.join(get_cached_logos_dir(), 'servers'), get_server_main_id_by_id(self.id),
+            LOGO_SIZE, LOGO_SIZE, keep_aspect_ratio=False, format='PNG'
+        )
+
     def save_session(self):
         """ Save session to disk """
 
@@ -335,8 +332,6 @@ class Server(BaseServer, ABC):
 
 
 def init_servers_modules(use_external_servers_modules, reload_modules=False):
-    clear_servers_finders()
-
     # Add a first Finder with HIGH priority
     server_finder = ServerFinder(priority=ServerFinderPriority.HIGH)
     server_finder.add_path(os.environ.get('KOMIKKU_SERVERS_PATH'))
@@ -357,9 +352,6 @@ def init_servers_modules(use_external_servers_modules, reload_modules=False):
         server_finder.add_path(os.path.join(get_cache_dir(), 'servers/repo'))
         server_finder.install()
 
-    if reload_modules:
-        get_servers_modules(reload=reload_modules)
-
 
 def install_servers_modules_from_repo(app_version):
     """
