@@ -1,5 +1,5 @@
-# Copyright (C) 2019-2024 Valéry Febvre
-# SPDX-License-Identifier: GPL-3.0-only or GPL-3.0-or-later
+# SPDX-FileCopyrightText: 2019-2025 Valéry Febvre
+# SPDX-License-Identifier: GPL-3.0-or-later
 # Author: Valéry Febvre <vfebvre@easter-eggs.com>
 
 from komikku.servers.multi.madara import Madara
@@ -7,10 +7,12 @@ from komikku.servers.multi.madara import Madara
 
 class Leomanga(Madara):
     id = 'leomanga'
-    name = 'Lector Manga (Leomanga)'
+    name = 'Lectormanga (Leomanga)'
     lang = 'es'
     is_nsfw = True
-    status = 'disabled'
+    has_cf = True
 
-    base_url = 'https://lectormanga.fun'
+    series_name = 'biblioteca'
+
+    base_url = 'https://lectormangaa.com'
     chapters_url = base_url + '/manga/{0}/ajax/chapters/'
